@@ -105,10 +105,10 @@
   # Then we need to start quartz() before starting rgl.
   # See https://github.com/dmurdoch/rgl/issues/27
   if (getOption("rgl.startQuartz", 
-         !onlyNULL && 
-         unixos == "Darwin" && 
-         .Platform$GUI != "AQUA") &&
-         interactive() && 
+           !onlyNULL &&
+           interactive() &&
+           unixos == "Darwin" && 
+           !(.Platform$GUI %in% c("AQUA", "RStudio"))) &&
          exists("quartz", getNamespace("grDevices"))) {
     grDevices::quartz()
     dev.off()
@@ -124,15 +124,14 @@
 
   if (!rgl.useNULL()) 
     setGraphicsDelay(unixos = unixos)
-  
-  registerInputHandler("shinyPar3d", convertShinyPar3d, force = TRUE)
-  registerInputHandler("shinyMouse3d", convertShinyMouse3d, force = TRUE)
 
   # handle pkgdown_print and fig_settings before they are in the CRAN version
 
   if (requireNamespace("pkgdown", quietly = TRUE)) {
-    if ("pkgdown_print" %in% getNamespaceExports("pkgdown"))
+    
+    if ("pkgdown_print" %in% getNamespaceExports("pkgdown")) {
       pkgdown_print <<- getExportedValue("pkgdown", "pkgdown_print")
+    }
     if ("fig_settings" %in% getNamespaceExports("pkgdown"))
       pkgdown_fig_settings <<- getExportedValue("pkgdown", "fig_settings")
   }         
@@ -197,8 +196,8 @@ rgl.init <- function(initValue = 0, onlyNULL = FALSE, debug = getOption("rgl.deb
 ##
 
 .onUnload <- function(libpath) {
-  removeInputHandler("shinyPar3d")
-  removeInputHandler("shinyMouse3d")
+  unregisterShinyHandlers()
+
   # shutdown
   .C( rgl_quit, success=FALSE )
   
